//! The RAq Approach: element.h:
/*!stores helper classes for functional purposes*/
//(c) 2006 Suzanne Matthews
/*
This file is part of the RaqApproach software package.

    RaqApproach is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RaqApproach is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RaqApproach; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef ELEMENTS_H_
#define ELEMENTS_H_

#include "common.h"

using namespace std;

//!this class stores the distance and corresponding taxa
/*!this class assists with the clustering process by storing a set of taxa
   and their associated distance*/
class element {
 public:
    //! copy constructor
    /*!takes in a string and a double and creates an element object out of them
        \param t holds taxa to be copied into element object
        \param d holds distance to be copied into element object
        \sa element()
        */
 	element(string t, double d);
 	//! default constructor
 	/*!creates an element object with null values
 	   \sa element(string t, double d)
 	   */
 	element();
 	//! get_taxa function
 	/*! \return the taxa associated with the element*/
 	string get_taxa() { return taxa; }
 	//! get distance function
 	/*! \return distance associated with an element*/
 	double get_dist() { return dist; }

 private:
 	double dist; //!< distance between two taxa
 	string taxa; //!< actual two taxa
};

element::element(string t, double d) { //!< copy constructor
	taxa = t;
	dist = d;
}

element::element() { //!< default constructor
	taxa = "";
	dist = 0;
}

//! helper class that assists with hashing procedure
/*! this class works closely with the merge section of code that merges common
taxa together*/
class helper {
public:
    //! copy constructor
    /*! takes in a vector<short> and a vector<int> to create a helper object
       \param i holds id of taxa or set of taxa
       \param t holds taxa or set of taxa
     */
	helper(vector<short> i, vector<int> t);
	//! default constructor
	helper();
	//! get id function
	/* \return id of taxa */
	vector<short> get_id() {return id;}
	//! get tree function
	/*! \return list of taxa associated with helper object*/
	vector<int> get_tree() { return taxa_list; }
	//! display function
	/*! displays both the id and taxa list associated with that id*/
	void display();

private:
	vector<short> id; //!< holds id of taxa or set of taxa
	vector<int> taxa_list; //!< holds taxa or set of taxa
};

helper::helper(vector<short>i, vector<int> t) { //!< copy constructor
	id = i;
	taxa_list = t;
}

helper::helper() { } //!< default constructor (empty)

void helper::display() { //!< displays id and associated taxa
	cout << "my id is: " << endl;
	cout << "| ";
	for ( unsigned int i = 0; i < id.size(); ++i ) {
		cout << id[i] << " | ";
	}
	cout << endl;
	cout << endl << "my contents are: " << endl;
	cout << ".";
	for ( unsigned int i = 0; i < taxa_list.size(); ++i ) {
		cout << taxa_list[i] << ".";
	}
	cout << endl << endl << endl;
}
#endif
